﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Machr_HledaniCest
{
    internal class City
    {
        public List<Junction> Junctions { get; set; }

        public List<Street> Streets { get; set; }


        public City()
        {
            Junctions = new List<Junction>();
            Streets = new List<Street>();
        }

        public Path FindFastersPath(Query query)
        {
            foreach (var t in query.TrafficChanges)
            {
                SetTrafficSituation(Streets.Where(s => s.Name == t.StreetName).First(), t.State);
            }

            Path result = GetMinimalPathBetween(Streets.Where(s => s.Name == query.StrartStreet).First(),
                Streets.Where(s => s.Name == query.EndStreet).First());

            return result;
        }

        public Path GetMinimalPathBetween(Street startStreet, Street endStreet)
        {

            List<Path> allPaths = new List<Path>();
            List<Street> streets = new List<Street>();

            streets.AddRange(Streets);
            foreach(Street s in streets)
            {
                allPaths.Add(new Path(startStreet, s));
            }

            double actualDistance = 0.0;
            Street actualStreet = startStreet;

            Path path = allPaths.Where(p => p.EndStreet.Name == actualStreet.Name).FirstOrDefault();
            path.Distance = actualDistance;

            streets.Remove(path.EndStreet);


            while(streets.Count > 0)
            {
                foreach(Street street in streets)
                {
                    double distance = 0.0;
                    if(!AreStreetsSidebySide(street, actualStreet))
                    {
                        distance = double.PositiveInfinity;
                    }
                    else
                    {
                        distance = actualDistance + ((actualStreet.ActualDistance / 2.0) + (street.ActualDistance / 2.0));
                    }

                    path = allPaths.Where(p => p.EndStreet.Name == street.Name).FirstOrDefault();
                    if (path.Distance > distance)
                    {
                        path.Distance = distance;

                        path.Streets = new List<Street>();
                        path.Streets.AddRange(allPaths.Where(p => p.EndStreet.Name == actualStreet.Name).First().Streets);
                        path.Streets.Add(street);
                    }

                }

                Path minPath = allPaths.Where(p => ((streets.Contains(p.EndStreet)))).OrderBy(p => p.Distance).First();

                if (minPath.EndStreet == endStreet)
                    break;

                streets.Remove(minPath.EndStreet);

                actualStreet = minPath.EndStreet;
                actualDistance = minPath.Distance;

            }

            Path result = allPaths.Where(p => p.EndStreet.Name == endStreet.Name).First();
            return result;

        }

        public bool AreStreetsSidebySide(Street s1, Street s2)
        {

            bool hasDirectPath = false;
            foreach(var j in s1.Junctions)
            {
                if (j.Streets.Contains(s2))
                {
                    hasDirectPath = true;
                    break;
                }
            }
            return hasDirectPath;
        }

        public void SetTrafficSituation(Street street, StreetState state)
        {
            Street targetStreet = Streets.Where(s => s.Name == street.Name).First();
            targetStreet.State = state;
        }

    }

    internal class Path
    {

        public Street StartStreet { get; set; }

        public Street EndStreet { get; set; }

        public List<Street> Streets { get; set; }

        public double Distance { get; set; }


        public Path()
        {
            Streets = new List<Street>();
            Distance = double.PositiveInfinity;
        }

        public Path(Street startStreet, Street endStreet)
        {
            Streets = new List<Street>();
            Distance = double.PositiveInfinity;
            StartStreet = startStreet;
            EndStreet = endStreet;
            Streets.Add(startStreet);
        }

        public override string ToString()
        {
            if (Distance == double.PositiveInfinity)
                return "-";

            NumberFormatInfo format = new NumberFormatInfo();
            format.NumberDecimalSeparator = ".";

            return $"{string.Join(" ", Streets)} {Distance.ToString(format)}";
        }

    }

}
