﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Machr_HledaniCest
{
    internal class Parser
    {

        public Street ParseStreet(string input)
        {
            NumberFormatInfo provider = new NumberFormatInfo();
            provider.NumberDecimalSeparator = ",";
            provider.NumberDecimalSeparator = ".";
            string[] array = input.Split(' ');

            Regex regex = new Regex(@"^[A-Za-z0-9_]+$");
            if (!(regex.IsMatch(array[0])))
            {
                throw new ArgumentException();
            }

            double length;
            if(!double.TryParse(array[1], NumberStyles.Float, provider, out length))
            {
                throw new ArgumentException();
            }

            return new Street(array[0], Convert.ToDouble(length, provider));
        }

        public void ParseJunction(string input, City graph)
        {
            Junction junction = new Junction();
            string[] array = input.Split(' ');
            foreach (string item in array)
            {
                Street street = graph.Streets.First(s => (s.Name == item));
                street.Junctions.Add(junction);
                junction.Streets.Add(street);
            }
            graph.Junctions.Add(junction);
        }

        public Tuple<string, string> ParseTrafficChange(string input)
        {
            string[] array = input.Split(' ');
            if(array.Count() == 2)
            {
                return new Tuple<string, string>(array[0], array[1]);
            }
            else
            {
                throw new ArgumentException();
            }
            
        }

        public bool CreateGraph(City city)
        {
            //ulice
            int streetsCount = 0;
            int junctionCount = 0;
            try
            {
                streetsCount = Convert.ToInt32(Console.ReadLine());
                junctionCount = Convert.ToInt32(Console.ReadLine());
            }
            catch (Exception)
            {
                return false;
            }

            string input;
            for (int i = 0; i < streetsCount; i++)
            {
                input = Console.ReadLine();
                try
                {
                    city.Streets.Add(ParseStreet(input));
                }
                catch (ArgumentException)
                {
                    return false;
                }
            }

            //křižovatky
            for (int i = 0; i < junctionCount; i++)
            {
                input = Console.ReadLine();
                ParseJunction(input, city);
            }

            return true;
        }

        public bool ParseQueries(List<Query> queries)
        {
            string input = string.Empty;
            input = Console.ReadLine();

            int queryCount;
            if(!Int32.TryParse(input, out queryCount))
            {
                Console.Clear();
                return false;
            }

            for (int i = 0; i < queryCount; i++)
            {
                Query query = new Query();

                input = Console.ReadLine();

                int changesCount;
                if(!Int32.TryParse(input, out changesCount))
                {
                    return false;
                }

                bool validQuery = true;

                //změny v dopravě
                for (int j = 0; j < changesCount; j++)
                {
                    input = Console.ReadLine();
                    try
                    {
                        Tuple<string, string> traffChange = ParseTrafficChange(input);
                        query.TrafficChanges.Add(new TraficChange(traffChange.Item2, traffChange.Item1));
                    }
                    catch (Exception)
                    {
                        return false;
                    }

                }

                if (validQuery == false)
                    continue;

                query.StrartStreet = Console.ReadLine();
                query.EndStreet = Console.ReadLine();

                queries.Add(query);
            }

            return true;
        }

    }
}
